var system = require("../../../system");
const AppServiceBase = require("../../app.base");

//此类用来db中配置来进行调用
class UtilsFqAliyunService extends AppServiceBase {
  constructor() {
    super();
  }
  async pushOrderBusinessByProductCode2Fq(pobj) {//推送自定义产品码的订单商机到峰擎（订单已经支付成功才推送）
    var verify = await this.verifyParam(pobj);
    var orderInfo = pobj.actionBody.order_info;
    var orderContact = pobj.actionBody.order_contact;
    var productInfo = pobj.actionBody.product_info;
    if (!orderInfo) {
      verify = system.getResult(null, "orderInfo can not be empty，100420");
    }
    if (!orderInfo.orderStatus || orderInfo.orderStatus < 2) {
      verify = system.getResult(null, "orderInfo status must be paid，100423");
    }
    if (!orderContact || !orderContact.mobile) {
      verify = system.getResult(null, "orderInfo.orderContact.mobile can not be empty，100425");
    }
    if (!productInfo || !productInfo.service_business_code) {
      verify = system.getResult(null, "orderInfo.productInfo.service_business_code can not be empty，100427");
    }
    if (!productInfo || !productInfo.price_item || !productInfo.price_item.service_code) {
      verify = system.getResult(null, "orderInfo.productInfo.price_item.service_code can not be empty，100430");
    }
    if (verify.status != 0) {
      this.pushlogSve.createDb({
        appid: pobj.appInfo.uapp_id,
        appkey: pobj.appInfo.uapp_key,
        requestId: pobj.requestId || "",
        op: "暂无推送的接口信息" + "，verify-msg：" + verify.msg,//推送的接口信息
        content: JSON.stringify(pobj),//推送的参数信息
        resultInfo: "",
        returnType: '0',
        opTitle: pobj.opType + "推送操作失败->opAliyunClientPost"
      });
      return verify;
    }

    var interface_params_info = verify.data;
    var params = {
      idempotentId: orderInfo.orderNo,// 是 业务 ID
      idempotentSource: productInfo.service_business_code,// 是 业务来源（ic_jdyun、ic_ali）
      idempotentSourceName: productInfo.service_business_name || productInfo.service_business_code,// 是 京东云应用、阿里云应用
      city: "",// 否 所属城市
      phone: orderContact.mobile,// 是 手机号
      userId: orderInfo.channelUserId,// 否 用户 ID
      companyName: "",// 否 公司名称
      orderPrice: orderInfo.totalSum,// 是 订单金额
      productId: productInfo.price_item.service_code,// 是 产品 ID
      productQuantity: orderInfo.quantity,// 是 产品数量
    };
    var result = this.opAliyunClientPost(pobj, "pushOrderBusiness", interface_params_info.opUrl, interface_params_info.key, interface_params_info.secret, params);
  }

  /**
   * 推送阿里信息操作到峰擎
   * 阿里操作的信息不变外加增加组装的一些参数（推送工商需求商机、推送工商订单商机、推送关闭工商需求）
   * @param {*} pobj 
   */
  async pushAliBusiness2Fq(pobj) {//推送阿里信息操作到峰擎
    var verify = await this.verifyParam(pobj);
    if (verify.status != 0) {
      this.pushlogSve.createDb({
        appid: pobj.appInfo.uapp_id,
        appkey: pobj.appInfo.uapp_key,
        requestId: pobj.requestId || "",
        op: "暂无推送的接口信息" + "，verify-msg：" + verify.msg,//推送的接口信息
        content: JSON.stringify(pobj),//推送的参数信息
        resultInfo: "",
        returnType: '0',
        opTitle: pobj.opType + "推送操作失败->opAliyunClientPost"
      });
      return verify;
    }
    var interface_params_info = verify.data;
    this.opAliyunClientPost(pobj, interface_params_info.opUrl, interface_params_info.key, interface_params_info.secret, pobj.actionBody);
  }
  async verifyParam(pobj) {//参数信息验证
    var verify = system.getResultSuccess();
    if (!pobj.interface_params) {
      verify = system.getResult(null, "interface_params can not be empty，100400");
    }
    var interface_params_info = JSON.parse(pobj.interface_params);
    if (!interface_params_info || !interface_params_info.opUrl || !interface_params_info.key || !interface_params_info.secret) {
      verify = system.getResult(null, "interface_params（opUrl、key、secret） can not be empty，100410");
    }
    verify.data = interface_params_info;
    return verify;
  }

  async opAliyunClientPost(pobj, url, key, secret, params) {
    try {
      var rc = system.getObject("util.aliyunClient");
      var rtn = await rc.post(url, key, secret, params);
      if (rtn.code != 200 && rtn.success != true) {
        this.pushlogSve.createDb({
          appid: pobj.appInfo.uapp_id,
          appkey: pobj.appInfo.uapp_key,
          requestId: pobj.requestId || "",
          op: JSON.stringify(pobj.interface_info),//推送的接口信息
          content: JSON.stringify(pobj),//推送的参数信息
          resultInfo: JSON.stringify(rtn),
          returnType: '0',
          opTitle: pobj.opType + "推送操作出错->opAliyunClientPost"
        });
        return system.getResult(null, "推送失败，失败原因：" + rtn.errorMsg + "，selfrequestId=" + pobj.requestId + "，requestId=" + rtn.requestId);
      }
      this.pushlogSve.createDb({
        appid: pobj.appInfo.uapp_id,
        appkey: pobj.appInfo.uapp_key,
        requestId: pobj.requestId || "",
        op: JSON.stringify(pobj.interface_info),//推送的接口信息
        content: JSON.stringify(pobj),//推送的参数信息
        resultInfo: JSON.stringify(rtn),
        returnType: '1',
        opTitle: pobj.opType + "数据推送成功->opAliyunClientPost"
      });
      return system.getResultSuccess(null, "推送成功");
    } catch (e) {
      this.pushlogSve.createDb({
        appid: pobj.appInfo.uapp_id,
        appkey: pobj.appInfo.uapp_key,
        requestId: pobj.requestId || "",
        op: JSON.stringify(pobj.interface_info),//推送的接口信息
        content: JSON.stringify(pobj),//推送的参数信息
        resultInfo: "error：" + e.stack,
        returnType: '0',
        opTitle: pobj.opType + "推送操作异常->opAliyunClientPost"
      });
      //日志记录
      this.logCtl.error({
        appid: pobj.appInfo ? pobj.appInfo.uapp_id || "" : "",
        appkey: pobj.appInfo ? pobj.appInfo.uapp_key || "" : "",
        requestId: pobj.requestId || "",
        op: "center-channel/app/base/service/impl/utilsSve/opAliyunClientPost.js/opAliyunClientPost",
        content: "error：" + e.stack,
        // clientIp: pobj.clientIp,
        optitle: pobj.opType + "推送操作异常->opAliyunClientPost",
      });
      return system.getResult(null, "推送error");
    }
  }
}
module.exports = UtilsFqAliyunService;
